VERSION 5.00
Begin VB.Form frmMain 
   Caption         =   "Form1"
   ClientHeight    =   3570
   ClientLeft      =   60
   ClientTop       =   345
   ClientWidth     =   6075
   LinkTopic       =   "Form1"
   ScaleHeight     =   3570
   ScaleWidth      =   6075
   StartUpPosition =   3  'Windows Default
End
Attribute VB_Name = "frmMain"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
'---------------------------------
'Title: DirectX Tutorial
'
'Description:
'
'Author: Jacob Roman
'
'Date: 12/02/2005
'
'Contact: jacob_c_roman@hotmail.com
'---------------------------------

Option Explicit

'The 2D (Transformed and Lit) vertex format type.
Private Type TLVERTEX

    X As Single
    Y As Single
    Z As Single
    RHW As Single
    Color As Long
    Specular As Long
    TU As Single
    TV As Single
    
End Type

'Some color depth constants to help make the DX constants more readable.
Private Const COLOR_DEPTH_16_BIT As Long = D3DFMT_R5G6B5
Private Const COLOR_DEPTH_24_BIT As Long = D3DFMT_A8R8G8B8
Private Const COLOR_DEPTH_32_BIT As Long = D3DFMT_X8R8G8B8

'The 2D (Transformed and Lit) vertex format.
Private Const FVF_TLVERTEX As Long = D3DFVF_XYZRHW Or D3DFVF_TEX1 Or D3DFVF_DIFFUSE Or D3DFVF_SPECULAR

Private DirectX8 As DirectX8 'The master DirectX object.
Private Direct3D As Direct3D8 'Controls all things 3D.
Private Direct3D_Device As Direct3DDevice8 'Represents the hardware rendering.
Private Direct3DX As D3DX8

Private Fullscreen_Enabled As Boolean 'Helps determine whether it's fullscreen mode.
Private Running As Boolean 'Helps determine whether the main game loop is running.

Private Vertex_List(3) As TLVERTEX '4 vertices will make a square.

Private Texture As Direct3DTexture8

Private Alpha As Long

'This function will make it much easier to setup the vertices with the info it needs.
Private Function Create_TLVertex(X As Single, Y As Single, Z As Single, RHW As Single, Color As Long, Specular As Long, TU As Single, TV As Single) As TLVERTEX

    Create_TLVertex.X = X
    Create_TLVertex.Y = Y
    Create_TLVertex.Z = Z
    Create_TLVertex.RHW = RHW
    Create_TLVertex.Color = Color
    Create_TLVertex.Specular = Specular
    Create_TLVertex.TU = TU
    Create_TLVertex.TV = TV
    
End Function

Private Function DirectX_Initialize() As Boolean

    On Error GoTo Error_Handler
    
    Dim Display_Mode As D3DDISPLAYMODE 'Display mode desciption.
    Dim Direct3D_Window As D3DPRESENT_PARAMETERS 'Backbuffer and viewport description.
    
    Set DirectX8 = New DirectX8 'Creates the DirectX object.
    Set Direct3D = DirectX8.Direct3DCreate() 'Creates the Direct3D object using the DirectX object.
    Set Direct3DX = New D3DX8
    
    If Fullscreen_Enabled = True Then
    
        'Now that we are working with fullscreen mode, we must set up the
        'screen resolution to switch to, rather than use the default screen
        'resolution.
        
        Display_Mode.Width = 800
        Display_Mode.Height = 600
        Display_Mode.Format = COLOR_DEPTH_16_BIT
    
        Direct3D_Window.Windowed = False 'The app will be in fullscreen mode.
        Direct3D_Window.BackBufferCount = 1 '1 backbuffer only
        Direct3D_Window.BackBufferWidth = Display_Mode.Width 'Match the backbuffer width with the display width
        Direct3D_Window.BackBufferHeight = Display_Mode.Height 'Match the backbuffer height with the display height
        Direct3D_Window.hDeviceWindow = frmMain.hWnd 'Use frmMain as the device window.
        
    Else
    
        Direct3D.GetAdapterDisplayMode D3DADAPTER_DEFAULT, Display_Mode 'Use the current display mode that you
                                                                        'are already on. Incase you are confused, I'm
                                                                        'talking about your current screen resolution. ;)
        
        Direct3D_Window.Windowed = True 'The app will be in windowed mode.
    
    End If
    
    Direct3D_Window.SwapEffect = D3DSWAPEFFECT_COPY_VSYNC 'Refresh when the monitor does.
    Direct3D_Window.BackBufferFormat = Display_Mode.Format 'Sets the format that was retrieved into the backbuffer.
    
    'Creates the rendering device with some useful info, along with the info
    'we've already setup for Direct3D_Window.
    Set Direct3D_Device = Direct3D.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, frmMain.hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, Direct3D_Window)
    
    Direct3D_Device.SetVertexShader FVF_TLVERTEX 'Set the type of vertex shading. (Required)
    
    'Right here will alphablend the polygon
    
    Direct3D_Device.SetTextureStageState 0, D3DTSS_COLOROP, D3DTOP_MODULATE
    Direct3D_Device.SetTextureStageState 0, D3DTSS_COLORARG1, D3DTA_TEXTURE
    Direct3D_Device.SetTextureStageState 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE
                   
    Direct3D_Device.SetTextureStageState 0, D3DTSS_ALPHAOP, D3DTOP_MODULATE
    Direct3D_Device.SetTextureStageState 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE
    Direct3D_Device.SetTextureStageState 0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE
    
    Direct3D_Device.SetRenderState D3DRS_SRCBLEND, D3DBLEND_SRCALPHA
    Direct3D_Device.SetRenderState D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA
    Direct3D_Device.SetRenderState D3DRS_BLENDOP, D3DBLENDOP_ADD
    
    'These lines are not needed, but it's nice to be able to filter the
    'textures to make them look nicer.
    
    Direct3D_Device.SetTextureStageState 0, D3DTSS_MINFILTER, D3DTEXF_POINT
    Direct3D_Device.SetTextureStageState 0, D3DTSS_MAGFILTER, D3DTEXF_POINT


    Exit Function
    
Error_Handler:
    
    MsgBox "An error occured while initializing DirectX", vbCritical
    Close_Program
    DirectX_Initialize = False


End Function

Private Sub Create_Polygon()

    Vertex_List(0) = Create_TLVertex(0, 0, 0, 1, D3DColorRGBA(255, 255, 255, Alpha), 0, 0, 0)
    Vertex_List(1) = Create_TLVertex(100, 0, 0, 1, D3DColorRGBA(255, 255, 255, Alpha), 0, 1, 0)
    Vertex_List(2) = Create_TLVertex(0, 100, 0, 1, D3DColorRGBA(255, 255, 255, Alpha), 0, 0, 1)
    Vertex_List(3) = Create_TLVertex(100, 100, 0, 1, D3DColorRGBA(255, 255, 255, Alpha), 0, 1, 1)

End Sub

Private Sub Load_Texture()

    Dim File_Path As String
    Dim Width As Long
    Dim Height As Long
    Dim Transparency_Color As Long
    
    File_Path = App.Path & "\Sprite1.png"

    Width = 256
    Height = 256
    
    Transparency_Color = D3DColorRGBA(0, 0, 0, 255)

    Set Texture = Direct3DX.CreateTextureFromFileEx(Direct3D_Device, _
                                                    File_Path, _
                                                    Width, Height, _
                                                    0, _
                                                    0, _
                                                    D3DFMT_A8R8G8B8, _
                                                    D3DPOOL_MANAGED, _
                                                    D3DX_FILTER_POINT, _
                                                    D3DX_FILTER_POINT, _
                                                    Transparency_Color, _
                                                    ByVal 0, _
                                                    ByVal 0)

End Sub

Private Sub Game_Loop()

    Do While Running = True
        
        DoEvents 'Allow events to happen so the program doesn't lock up.
        
        frmMain.Caption = "Press A/Z to change alpha values. Alpha: " & Alpha
        
        '----------------------------------------------------
        'DirectX automatically handles the framerate for you
        'which makes it run (at most) as fast as the monitors
        'refresh rate, so you don't need to add extra code to
        'slow down the loop and run at a certain number of frames
        'per second.
        '----------------------------------------------------
        
        'Clears the backbuffer.
        Direct3D_Device.Clear 0, ByVal 0, D3DCLEAR_TARGET, D3DColorRGBA(0, 255, 0, 0), 1#, 0
            
            Direct3D_Device.BeginScene
            
                'Right here will alphablend the polygon
                Direct3D_Device.SetRenderState D3DRS_ALPHABLENDENABLE, True
                
                Create_Polygon
                
                'Set the texture
                Direct3D_Device.SetTexture 0, Texture
            
                'Draw the polygon
                Direct3D_Device.DrawPrimitiveUP D3DPT_TRIANGLESTRIP, 2, Vertex_List(0), Len(Vertex_List(0))
            
            Direct3D_Device.EndScene
        
        'Flips the backbuffer into the form window.
        Direct3D_Device.Present ByVal 0, ByVal 0, 0, ByVal 0
        
    Loop

End Sub

Private Sub Close_Program()

    Running = False 'This helps the program bail out of the game loop.
    
    'Unload all of the DirectX objects.
    
    Set Texture = Nothing
    Set Direct3DX = Nothing
    Set Direct3D_Device = Nothing
    Set Direct3D = Nothing
    Set DirectX8 = Nothing
    
    Unload Me 'Unload the form.
    
    End 'Ends the program.
    
    'Although the Unload statement located above exits the program, you
    'will end up with an Automation error after doing so. The End statement
    'will help prevent that, and end the app completely.

End Sub

Private Sub Form_KeyDown(KeyCode As Integer, Shift As Integer)

    If KeyCode = vbKeyEscape Then 'If the user presses the Esc key...
        Close_Program
    End If
    
    If KeyCode = vbKeyA Then Alpha = Alpha + 2
    If KeyCode = vbKeyZ Then Alpha = Alpha - 2
    
    If Alpha >= 255 Then Alpha = 255
    If Alpha <= 0 Then Alpha = 0

End Sub

Private Sub Form_Load()

    'This event will fire before the form has completely loaded.
    
    If MsgBox("Click Yes to go to full screen (Recommended)", vbQuestion Or vbYesNo, "Options") = vbYes Then Fullscreen_Enabled = True
    Alpha = 255
    frmMain.Show
    frmMain.Caption = "DirectX Tutorial"
    DirectX_Initialize 'Initializes DirectX and Direct3D.
    Create_Polygon 'Creates the polygon.
    Load_Texture 'Loads a texture from file.
    Running = True 'Initializations all set. It's now ok to activate the game loop.
    Game_Loop

End Sub

Private Sub Form_Unload(Cancel As Integer)

    Close_Program

End Sub
